@@ -8,7 +8,7 @@ use Moo;
 use namespace::autoclean;
 use 5.10.1;
 
-our $VERSION = '0.1.6';
+our $VERSION = '0.1.7';
 
 BEGIN {
     if (my $trace_var = $ENV{PERL_ZOOKEEPER_TRACE}) {
@@ -29,6 +29,12 @@ ZooKeeper - Perl bindings for Apache ZooKeeper
     my @children = $zk->get_children('/', watcher => sub { my $event = shift; $cv->send($event) });
     my $child_event = $cv->recv;
 
+=head1 STATUS
+
+Unstable.
+
+Until version 1.0.0, some aspects of the API may change, most likely related to exception handling for commands and watchers.
+
 =head1 DESCRIPTION
 
 ZooKeeper is a perl interface to the Apache ZooKeeper C client library.
@@ -175,7 +181,7 @@ after watcher => sub {
     my @old = $self->get_watchers(%watcher_args);
     my $new = $self->create_watcher($path => $code, type => $type);
     $self->_set_watcher($new);
-    $self->remove_watcher(%watcher_args, watcher => $_) for @old;
+    $self->remove_watcher($_) for @old;
 };
 
 
@@ -208,6 +214,15 @@ The client_id for a ZooKeeper session. Can be set during construction to resume
 
 =cut
 
+=head2 default_acl
+
+=cut
+
+has default_acl => (
+    is      => 'ro',
+    default => sub { ZOO_OPEN_ACL_UNSAFE },
+);
+
 =head2 dispatcher
 
 The implementation of ZooKeeper::Dispatcher to be used. Defaults to AnyEvent.
@@ -267,8 +282,19 @@ sub BUILD {
         $self->create_watcher('' => $self->watcher, type => 'default');
     };
 
-    $self->_xs_init($self->hosts, $self->timeout, $watcher, $args->{client_id});
-    $self->add_auth(@{$self->authentication||[]}) if $self->authentication;
+    $self->_xs_init;
+    $self->connect(
+        authentication => $self->authentication,
+        client_id      => $args->{client_id},
+        hosts          => $self->hosts,
+        timeout        => $self->timeout,
+        watcher        => $watcher,
+    );
+}
+
+sub DEMOLISH {
+    my ($self) = @_;
+    $self->_xs_destroy;
 }
 
 =head1 METHODS
@@ -302,6 +328,42 @@ Can optionally be passed a timeout(specified in seconds), which will cause wait
 
     OPTIONAL $seconds
 
+=cut
+
+around connect => sub {
+    my ($orig, $self, %args) = @_;
+    my ($hosts, $timeout, $watcher, $client_id, $authentication) =
+        @args{qw(hosts timeout watcher client_id authentication)};
+    $self->$orig($hosts, $timeout, $watcher, $client_id);
+    $self->add_auth(@$authentication) if $authentication;
+};
+
+=head2 close
+
+Close a ZooKeeper session.
+
+If the handle was not created by the current process, a ZOO_CLOSE_OP will NOT be sent to the server. Instead, only the underlying socket will be closed.
+
+=head2 reopen
+
+Reopen a ZooKeeper session after forking.
+
+This creates a new ZooKeeper session, without closing the parent session.
+
+=cut
+
+sub reopen {
+    my ($self, %args) = @_;
+    $self->close;
+    # since this creates a new session, don't pass a client_id
+    $self->connect(
+        authentication => $self->authentication,
+        hosts          => $self->hosts,
+        timeout        => $self->timeout,
+        watcher        => $self->watcher,
+        %args,
+    );
+}
 
 =head2 create
 
@@ -322,6 +384,7 @@ Create a new node with the given path and data. Returns the path for the newly c
 
 around create => sub {
     my ($orig, $self, $path, %extra) = @_;
+    my $acl   = $extra{acl} // $self->default_acl;
     my $flags = 0;
     $flags |= ZOO_EPHEMERAL if $extra{ephemeral};
     $flags |= ZOO_SEQUENCE  if $extra{sequential};
@@ -331,7 +394,7 @@ around create => sub {
         $path,
         $extra{value},
         $extra{buffer_length} // $self->buffer_length,
-        $extra{acl}           // ZOO_OPEN_ACL_UNSAFE,
+        $acl,
         $flags,
     );
 };
@@ -376,6 +439,19 @@ around delete => sub {
     return $self->$orig($path, $extra{version}//-1);
 };
 
+=head2 ensure_path
+
+=cut
+
+sub ensure_path {
+    my ($self, $path, %extra) = @_;
+    return if $self->exists($path);
+
+    my ($parent) = $path =~ m#^(.*)/[^/]+$#;
+    $self->ensure_path($parent, %extra) if $parent;
+    $self->create($path, %extra);
+}
+
 =head2 exists
 
 Check whether a node exists at the given path, and optionally set a watcher for when the node is created or deleted.
@@ -435,7 +511,11 @@ In list context, the data and stat hashref of the node is returned. Otherwise ju
 around get => sub {
     my ($orig, $self, $path, %extra) = @_;
     my $watcher = $extra{watcher} ? $self->create_watcher($path, $extra{watcher}, type => 'get') : undef;
-    return $self->$orig($path, $extra{buffer_length}//$self->buffer_length, $watcher);
+    return $self->$orig(
+        $path,
+        $extra{buffer_length} // $self->buffer_length,
+        $watcher,
+    );
 };
 
 =head2 set
@@ -519,7 +599,7 @@ Set the tracing level for the ZooKeeper client. Can also be set using the PERL_Z
 
 =head2 Forking
 
-The underlying C library uses POSIX threads. This means that ZooKeeper is not fork safe. Only exec and POSIX::_exit can be guaranteed to work safely in the child process.
+ZooKeeper now offers experimental support for forking safely. A child process may use either the close or reopen methods on a handle, after forking, to destroy the previous connection. Since forking in a multithreaded process is usually very dangerous, this library only closes the underlying socket, and removes references to the previous zhandle.
 
 =head2 Signals
 
